# -*- coding: utf-8 -*-
# @Author  : lx

import random

def copy_headers_dict(headers_raw):
    """ headers Converts
    Example:
     copy_headers = '''
          accept-encoding: gzip
          accept-language: zh-CN
          upgrade-insecure-requests: 1
          user-agent: Mozilla/5.0 Safari/537.36
        '''
     print(headers_raw_to_dict(copy_headers))
    """
    if headers_raw is None:
        return None

    headers = headers_raw.splitlines()
    headers_tuples = [header.split(":", 1) for header in headers]

    result_dict = {}
    for header_item in headers_tuples:
        if not len(header_item) == 2:
            continue

        item_key = header_item[0].strip()
        item_value = header_item[1].strip()
        result_dict[item_key] = item_value

    return result_dict


def get_ua():
    """Get some user-agent ,But not necessarily accepted by the website"""
    first_num = random.randint(55, 62)
    third_num = random.randint(0, 3200)
    fourth_num = random.randint(0, 140)
    os_type = [
        '(Windows NT 6.1; WOW64)',
        '(Windows NT 10.0; WOW64)',
        '(X11; Linux x86_64)',
        '(Macintosh; Intel Mac OS X 10_12_6)'
    ]
    chrome_version = 'Chrome/{}.0.{}.{}'.format(first_num, third_num, fourth_num)
    ua = ' '.join(['Mozilla/5.0', random.choice(os_type), 'AppleWebKit/537.36',
                   '(KHTML, like Gecko)', chrome_version, 'Safari/537.36']
                  )
    userAgent = {"user-agent":ua}
    return userAgent



# -*- coding: utf-8 -*-
# @Author  : lx

import re
from lxml import etree
from urllib import parse


def html_format(string):
    """ html去除标签 """
    dr = re.compile(r'<[^>]+>', re.S)
    not_format = dr.sub('', string)
    return not_format


def jsonp_to_json(jsonp):
    result = re.findall(r'\w+[(]{1}(.*)[)]{1}',jsonp,re.S)
    return result


def re_xpath(node,compile):
    """
    :param compile: './/span[re:match(@class, "allstar(\d0)")]/@class'
    """
    namespaces = {"re": "http://exslt.org/regular-expressions"}
    result = node.xpath(compile, namespaces=namespaces)
    return result


def url_parse(url:str):
    # 提取url中的params参数，返回item
    p = url.split('?')[1]
    item = {}
    if '&' in p:
        param = p.split('&')
        for v in param:
            k = v.split('=')
            item[k[0]] = k[1]
    else:
         k = p.split('=')
         item[k[0]] = k[1]
    return item



def change_body_resource(body,domain,types="IMAGE",mark=False):
    # 替换路径为绝对路径、添加标记
    types_item = {
        'IMAGE':'//img/@src',
        'FILE':'//a[contains(@href, ".pdf")]/@href',
        'VIDEO':'//a[contains(@href, ".mp4") or contains(@href, ".avi") or contains(@href, ".mkv")]/@href',
        'AUDIO':'//a[contains(@href, ".mp3") or contains(@href, ".wav")]/@href',
    }
    body = body.replace(' data-src=',' src=')
    doc = etree.HTML(body)
    label = 1
    for img in doc.xpath(types_item[types]):
        if img.startswith('data:image'):
            continue
        if not img.startswith('http'):
            p_img = parse.urljoin(domain,img)
            if not p_img.startswith('http'):
                p_img = domain+p_img
        else:
            p_img = img
        if '?' in p_img:
            # parse &amp;
            p_img = p_img.split('?')[0]
            img = img.split('?')[0]
        body = body.replace(img,p_img)
        # parse mark
        if mark:
            label_text = ''.join(re.findall(p_img+".*?\">",body,re.S))
            body = body.replace(label_text,label_text+"\n{%s:%s}"%(types,label))
            label+=1
    return body



# -*- coding: utf-8 -*-
# @Author  : lx
# @IDE ：PyCharm

import hmac
import hashlib
# hmac
def get_hamc(key,text,model=hashlib.sha256):
    key = key.encode()
    texts = text.encode()
    mac = hmac.new(key, texts, model)
    mac.digest()
    return mac.hexdigest()



# -*- coding: utf-8 -*-
# @Time    : 2021/12/28 12:29
# @Author  : lx
# @IDE ：PyCharm


import base64
from Crypto.Cipher import AES
# AES
# 需要补位，str不是16的倍数那就补足为16的倍数
def add_to_16(value):
    while len(value) % 16 != 0:
        value += '\0'
    return str.encode(value)  # 返回bytes

# 加密方法
def aes_encrypt(key, text):
    aes = AES.new(add_to_16(key), AES.MODE_ECB)  # 初始化加密器
    encrypt_aes = aes.encrypt(add_to_16(text))  # 先进行aes加密
    encrypted_text = str(base64.encodebytes(encrypt_aes), encoding='utf-8')
    return encrypted_text

# 解密方法
def aes_decrypt(key, text):
    aes = AES.new(add_to_16(key), AES.MODE_ECB)  # 初始化加密器
    base64_decrypted = base64.decodebytes(text.encode(encoding='utf-8'))
    decrypted_text = str(aes.decrypt(base64_decrypted), encoding='utf-8').replace('\0', '')  # 执行解密密并转码返回str
    return decrypted_text





# -*- coding: utf-8 -*-
# @Time    : 2021/12/28 12:29
# @Author  : lx
# @IDE ：PyCharm

import binascii
from pyDes import des, CBC, PAD_PKCS5

# DES
def des_encrypt(secret_key, s):
    iv = secret_key
    k = des(secret_key, CBC, iv, pad=None, padmode=PAD_PKCS5)
    en = k.encrypt(s, padmode=PAD_PKCS5)
    return binascii.b2a_hex(en)

def des_decrypt(secret_key, s):
    iv = secret_key
    k = des(secret_key, CBC, iv, pad=None, padmode=PAD_PKCS5)
    de = k.decrypt(binascii.a2b_hex(s), padmode=PAD_PKCS5)
    return de

#secret_str = des_encrypt('999', 'lx-message')
#clear_str = des_decrypt('999', secret_str)



# -*- coding: utf-8 -*-
# @Time    : 2021/12/28 12:34
# @Author  : lx
# @IDE ：PyCharm

import base64
def rc4(text, key = b'default-key', mode = "encode"):
    if mode == "decode": text = base64.b64decode(text)
    S = list(range(256))
    j = 0
    for i in range(256):
        j = (j + S[i] + key[i%len(key)]) % 256
        S[i], S[j] = S[j], S[i]
    i, j = 0, 0
    R = []
    for c in text:
        i = (i + 1) % 256
        j = (j + S[i]) % 256
        S[i], S[j] = S[j], S[i]
        t = c ^ (S[(S[i] + S[j]) % 256])
        R.append(t)
    if mode == "encode": return base64.b64encode(bytes(R))
    return bytes(R)

#text  = '123'.encode()
#key = '123'.encode()
#encd = rc4(text,key,mode='encode');print(encd)
#decd = rc4(encd,key,mode='decode');print(decd)


#-*- coding:utf-8 -*-
# @Author  : lx


"""
解决 etree.HTML解析后数据丢失的问题
调用 create_root_node 方法，传入response.text即可
"""

from lxml import etree, html


class SafeXMLParser(etree.XMLParser):
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('resolve_entities', False)
        super(SafeXMLParser, self).__init__(*args, **kwargs)


_ctgroup = {
    'html': {
        '_parser': html.HTMLParser,
        '_tostring_method': 'html',
    },
    'xml': {
        '_parser': SafeXMLParser,
        '_tostring_method': 'xml',
    },
        }
_default_type = None


def _st(st):
    if st is None:
        return 'html'
    elif st in _ctgroup:
        return st
    else:
        raise ValueError('Invalid type: %s' % st)


def create_root_node(text, base_url=None, doc_type='html'):
    """Create root node for text using given parser class.
    """
    st = _st(doc_type or _default_type)
    parser_cls = _ctgroup[st]['_parser']
    body = text.strip().replace('\x00', '').encode('utf-8') or b'<html/>'
    parser = parser_cls(recover=True, encoding='utf-8')
    root = etree.fromstring(body, parser=parser, base_url=base_url)
    if root is None:
        root = etree.fromstring(b'<html/>', parser=parser, base_url=base_url)
    return root




#-*- coding:utf-8 -*-
# @Author  : lx

from setuptools import setup, find_packages

with open("README.rst", "r") as f:
  long_description = f.read()

setup(
    name = 'lxpy',
    url='https://github.com/lixi5338619/lxpy.git',
    version = '1.3.9',
    description='Web crawler and data processing toolkit !',
    long_description=long_description,
    #packages = find_packages(),
    packages = ['lxpy','lxpy/encrypt'],
    author = 'ying5338619',
    author_email = '125066648@qq.com',
    platforms = ["all"],
    install_requires=["wheel"],
    )



# python setup.py sdist bdist_wheel
# twine upload dist/*

